@@ -1,19 +1,32 @@
 
+.check.cpp.error <- function(r) {
+  # We expect a numeric matrix is returned. If a string is returned, then it is the error message.
+  if (is.character(r)) {
+    stop(r)
+  } else if (is.numeric(r)) {
+    r
+  } else stop("Unknown top of the returned object from cpp")
+}
+
 .supc1.cpp2 <- function(x, parameters, tolerance, verbose) {
+  .check.compatibility()
   stopifnot(length(parameters$tau) == length(parameters$t))
   lapply(seq_along(parameters$tau), function(i) {
     .current.tau <- parameters$tau[i]
     .current.t <- parameters$t[[i]]
-    .supc1.cpp2.internal(x, .current.tau, .current.t, tolerance, verbose)
+    r <- .supc1.cpp2.internal(x, .current.tau, .current.t, tolerance, verbose)
+    .check.cpp.error(r)
   })
 }
 
 .supc1.cpp <- function(x, parameters, tolerance, verbose) {
+  .check.compatibility()
   stopifnot(length(parameters$tau) == length(parameters$t))
   lapply(seq_along(parameters$tau), function(i) {
     .current.tau <- parameters$tau[i]
     .current.t <- parameters$t[[i]]
-    .supc1.cpp.internal(x, .current.tau, .current.t, tolerance, .dist, verbose)
+    r <- .supc1.cpp.internal(x, .current.tau, .current.t, tolerance, .dist, verbose)
+    .check.cpp.error(r)
   })
 }
 
@@ -34,6 +47,8 @@
         }
       } else d <- .dist(x)
       .T <- .current.t(t)
+      if (is.character(.T)) stop(.T)
+      if (.T <= 0) stop("invalid T(t)")
       t <- t + 1
       f <- exp(-d / .T)
       f[d > .current.tau] <- 0
@@ -113,9 +128,26 @@
   } else {
     stop("Invalid parameter t")
   }
+  # add safety
+  retval$t <- lapply(retval$t, .safety_wrapper)
   retval
 }
 
+.safety_wrapper <- function(f) {
+  f <- force(f)
+  function(t) {
+    tryCatch({
+      r <- f(t[1])
+      if (!is.numeric(r)) stop(sprintf("T(%d) is not numeric", t[1]))
+      if (length(r) != 1) stop(sprintf("T(%d) is not a scalar", t[1]))
+      if (r <= 0) stop(sprintf("T(%d) is not positive", t[1]))
+      r
+    }, error = function(e) {
+      conditionMessage(e)
+    })
+  }
+}
+
 #'@title Self-Updating Process Clustering
 #'
 #'@description 
@@ -157,7 +189,6 @@
 #'\item{result}{The position of data after iterations.}
 #'
 #'@examples
-#'\dontrun{
 #'set.seed(1)
 #'X <- local({
 #'  mu <- list(
@@ -183,7 +214,7 @@
 #'  }
 #'  X
 #'})
-#'X.supcs <- supc1(X, r = c(0.9, 1.7, 2.5), t = "dynamic")
+#'X.supcs <- supc1(X, r = c(0.9, 1.7, 2.5), t = "dynamic", implementation = "R")
 #'X.supcs$cluster
 #'plot(X.supcs[[1]], type = "heatmap", major.size = 2)
 #'plot(X.supcs[[2]], type = "heatmap", col = cm.colors(24), major.size = 5)
@@ -191,10 +222,9 @@
 #'X.supcs <- supc1(X, r = c(1.7, 2.5), t = list(
 #'  function(t) {1.7 / 20 + exp(t) * (1.7 / 50)},
 #'  function(t) {exp(t)}
-#'))
+#'), implementation = "R")
 #'plot(X.supcs[[1]], type = "heatmap", major.size = 2)
 #'plot(X.supcs[[2]], type = "heatmap", col = cm.colors(24), major.size = 5)
-#'}
 #'
 #'@references
 #'Shiu, Shang-Ying, and Ting-Li Chen. 2016. "On the Strengths of the Self-Updating Process Clustering Algorithm." Journal of Statistical Computation and Simulation 86 (5): 1010–1031. \doi{10.1080/00949655.2015.1049605}.
@@ -299,8 +329,8 @@ supc1 <- function(
 #'\item{result}{The position of data after iterations.}
 #'
 #'@examples
-#'\dontrun{
 #'# The shape data has a structure of five clusters and a number of noise data points.
+#'\donttest{
 #'makecircle=function(N, seed){
 #'  n=0
 #'  x=matrix(NA, nrow=N, ncol=2)
@@ -334,11 +364,11 @@ supc1 <- function(
 #'  return(x)
 #'}
 #'
-#'shape1 <- makedata(5000, 1000)
+#'shape1 <- makedata(250, 100)
 #'dim(shape1)
 #'plot(shape1)
 #'
-#'X.supc=supc.random(shape1, r=0.5, t="dynamic", k = 500)
+#'X.supc=supc.random(shape1, r=0.5, t="dynamic", k = 500, implementation = "R")
 #'plot(shape1, col=X.supc$cluster)
 #'}
 #'
@@ -415,6 +445,7 @@ supc.random <- function(
 }
 
 .supc.random.cpp <- function(x, parameters, tolerance, verbose) {
+  .check.compatibility()
   stopifnot(length(parameters$tau) == length(parameters$t))
   lapply(seq_along(parameters$tau), function(i) {
     .current.tau <- parameters$tau[i]
@@ -426,7 +457,8 @@ supc.random <- function(
       stopifnot(is.list(groups))
     }
     k <- parameters$k[i]
-    .supc.random.cpp.internal(x, .current.tau, .current.t, k, groups, tolerance, verbose)
+    r <- .supc.random.cpp.internal(x, .current.tau, .current.t, k, groups, tolerance, verbose)
+    .check.cpp.error(r)
   })
 }
 
@@ -451,6 +483,8 @@ supc.random <- function(
         }
       }
       .T <- .current.t(t)
+      if (is.character(.T)) stop(.T)
+      if (.T <= 0) stop("Invalid T(t)")
       t <- t + 1
       if (t > length(groups)) {
         groups[[t]] <- .group.idx[sample(nrow(x))]
@@ -489,6 +523,7 @@ supc.random <- function(
 #'@description
 #'Plot the frequency polygon of the pairwise distance.
 #'@aliases freq.poly.default freq.poly.dist
+#'@inherit graphics::hist return
 #'@export
 freq.poly <- function(x, ...) {
   UseMethod("freq.poly")
@@ -519,6 +554,7 @@ freq.poly.dist <- function(x, ...) {
 #'@description
 #'Plot the frequency polygon of the pairwise distance. The red dashed line is the used parameter \eqn{r}.
 #'@aliases freq.poly.subclist
+#'@return \code{NULL}. The function is called for side effects.
 #'@export
 freq.poly.supc <- function(x, ...) {
   if (is.null(x$d0)) x$d0 <- .dist(x$x)
@@ -554,11 +590,11 @@ freq.poly.supclist <- function(x, ...) {
 #'  \item{\code{"heatmap"}}{draw a heatmap to show the result of clustering. The clusters whose size is greater than parameter \code{major.size} are treated as major clusters.}
 #'}
 #'@param ... other parameters to be passed through.
-#'
+#'@return \code{NULL}. The function is called for side effects.
 #'@examples
-#'\dontrun{
+#'\donttest{
 #'data(golub, package = "supc")
-#'golub.supc <- supc1(golub, rp = 0.0005, t = "dynamic")
+#'golub.supc <- supc1(golub, rp = 0.0005, t = "dynamic", implementation = "R)
 #'table(golub.supc$size)
 #'plot(golub.supc, type = "heatmap", major.size = 10)
 #'}
@@ -600,4 +636,13 @@ heatmap.supc <- function(x, ..., major.size = 1, yaxt = "n", xlab = "Samples", y
     }
   }
   graphics::axis(side = 3, at = c(major.at, minor.at), labels = c(major.label, minor.size), tick = FALSE, mgp = c(1.5, 0, 0), padj = -0.5)
+  invisible(NULL)
+}
+
+.check.compatibility.error.msg <- "The implementation in cpp is not supported on Solaris"
+
+.check.compatibility <- function() {
+  if (Sys.info()[["sysname"]] == "SunOS") {
+    stop(.check.compatibility.error.msg)
+  }
 }
