@@ -50,9 +50,12 @@ int cblas_Rdgemm(const enum CBLAS_ORDER Order, const enum CBLAS_TRANSPOSE TransA
          F77_TA = C2F_CHAR(&TA);
          F77_TB = C2F_CHAR(&TB);
       #endif
+      #ifndef FCONE
+         # define FCONE
+      #endif
 
       F77_CALL(dgemm)(F77_TA, F77_TB, &F77_M, &F77_N, &F77_K, &alpha, A,
-       &F77_lda, B, &F77_ldb, &beta, C, &F77_ldc);
+       &F77_lda, B, &F77_ldb, &beta, C, &F77_ldc FCONE FCONE);
       return 0;
    }
    return 1;
